@@ -11,7 +11,7 @@ use Date::Calc qw(Day_of_Week check_date);
 use List::MoreUtils qw(all any);
 use Params::Validate ':all';
 
-our $VERSION = '0.73';
+our $VERSION = '0.75';
 
 validation_options(
     on_fail => sub
@@ -113,7 +113,7 @@ sub parse_datetime
     if (scalar keys %count == 1 && $count{(keys %count)[0]} == 2) {
         if ($date_string =~ /^\S+\b\s+\b\S+/) {
             ($date_string, @{$self->{tokens}}) = split /\s+/, $date_string;
-            $self->{count}{tokens} = 1 + @{$self->{tokens}};
+            $self->{count}{tokens} = 1 + scalar @{$self->{tokens}};
         }
         else {
             $self->{count}{tokens} = 1;
@@ -128,7 +128,14 @@ sub parse_datetime
         my $format = $self->{Format};
 
         if (exists $length{4}) {
-            $format = join $separator, ($length{4} == 0 ? qw(yyyy mm dd) : qw(dd mm yyyy));
+            $format = join $separator,
+              ($length{4} == 0
+                ? qw(yyyy mm dd)
+                : ($format =~ /^m/
+                    ? qw(mm dd yyyy)
+                    : qw(dd mm yyyy)
+                  )
+              );
         }
         else {
             $separator = do { local $_ = $format;
@@ -175,6 +182,7 @@ sub parse_datetime
         $self->_set_modified(1);
 
         if (@{$self->{tokens} || []}) {
+            $self->{count}{tokens}--;
             $self->_unset_valid_exp;
             $self->_unset_modified;
             $self->_process;
@@ -231,12 +239,12 @@ sub parse_datetime_duration
       ? split /\s+ $timespan_sep \s+/ix, $self->{Date_string}
       : ($self->{Date_string});
 
-    my @stack;
+    my @queue;
     foreach my $date_string (@date_strings) {
-        push @stack, $self->parse_datetime($date_string);
+        push @queue, $self->parse_datetime($date_string);
     }
 
-    return @stack;
+    return @queue;
 }
 
 sub success
@@ -262,25 +270,27 @@ sub trace
 {
     my $self = shift;
 
-    my @modified;
-    foreach my $unit (grep { $_ ne 'total' } keys %{$self->{modified}}) {
-        push @modified, "$unit: $self->{modified}{$unit}";
-    }
-
-    return join "\n", @{$self->{trace}}, @modified;
+    return join "\n", @{$self->{trace}},
+      map  { my $unit = $_; "$unit: $self->{modified}{$unit}" }
+      grep { $_ ne 'total' } keys %{$self->{modified}};
 }
 
 sub _process
 {
     my $self = shift;
 
-    PARSE: foreach my $keyword (keys %{$self->{data}->__grammar('')}) {
+    if (!exists $self->{lookup}) {
+        foreach my $keyword (keys %{$self->{data}->__grammar('')}) {
+            push @{$self->{lookup}{scalar @{$self->{data}->__grammar($keyword)->[0]}}}, $keyword;
+        }
+    }
+
+    PARSE: foreach my $keyword (@{$self->{lookup}{$self->{count}{tokens}} || []}) {
+        last if $self->_get_modified >= $self->{count}{tokens};
+
         my @grammar = @{$self->{data}->__grammar($keyword)};
         my $types = shift @grammar;
 
-        next if @{$self->{tokens}} > @$types;
-        last if $self->_get_modified >= @{$self->{tokens}};
-
         foreach my $expression (@grammar) {
             my $valid_expression = true;
             my $definition = $expression->[0];
@@ -294,6 +304,7 @@ sub _process
                         }
                         else {
                             $valid_expression = false;
+                            last;
                         }
                     }
                 }
@@ -305,6 +316,7 @@ sub _process
                     }
                     else {
                         $valid_expression = false;
+                        last;
                     }
                 }
                 else {
@@ -312,10 +324,17 @@ sub _process
                         "unknown type $types->[$pos]\n";
                 }
             }
+            if ($valid_expression && @{$expression->[2]}) {
+                my $i;
+                foreach my $check (@{$expression->[2]}) {
+                    my @pos = @{$expression->[1][$i++]};
+                    $valid_expression &= $check->(\%regex_stack, @pos);
+                }
+            }
             if ($valid_expression) {
                 $self->_set_valid_exp;
                 my $i;
-                foreach my $positions (@{$expression->[1]}) {
+                foreach my $positions (@{$expression->[3]}) {
                     my ($c, @values);
                     foreach my $pos (@$positions) {
                         $values[$c++] = exists $regex_stack{$pos}
@@ -398,28 +417,16 @@ sub _unset_modified  { $_[0]->{modified}{total}  = 0     }
 sub _get_datetime_object
 {
     my $self = shift;
-
-    $self->{Time_zone} = $self->{datetime}->time_zone->name;
-    $self->{year}      = $self->{datetime}->year;
-    $self->{month}     = $self->{datetime}->month;
-    $self->{day}       = $self->{datetime}->day_of_month;
-    $self->{hour}      = $self->{datetime}->hour;
-    $self->{min}       = $self->{datetime}->minute;
-    $self->{sec}       = $self->{datetime}->second;
-
-    $self->{sec}       = sprintf("%02d", $self->{sec});
-    $self->{min}       = sprintf("%02d", $self->{min});
-    $self->{hour}      = sprintf("%02d", $self->{hour});
-    $self->{day}       = sprintf("%02d", $self->{day});
-    $self->{month}     = sprintf("%02d", $self->{month});
-
-    my $dt = DateTime->new(time_zone => $self->{Time_zone},
-                           year      => $self->{year},
-                           month     => $self->{month},
-                           day       => $self->{day},
-                           hour      => $self->{hour},
-                           minute    => $self->{min},
-                           second    => $self->{sec});
+    
+    my $dt = DateTime->new(
+        time_zone => $self->{datetime}->time_zone->name,
+        year      => $self->{datetime}->year,
+        month     => $self->{datetime}->month,
+        day       => $self->{datetime}->day_of_month,
+        hour      => $self->{datetime}->hour,
+        minute    => $self->{datetime}->minute,
+        second    => $self->{datetime}->second,
+    );
 
     foreach my $unit (keys %{$self->{postprocess}}) {
         $dt->add("${unit}s" => $self->{postprocess}{$unit});
@@ -507,7 +514,7 @@ Specifies the format of numeric dates, defaults to 'C<d/m/y>'.
 
 =item * C<prefer_future>
 
-Turns ambigious weekdays/months to their futuristic relatives. Accepts a boolean,
+Turns ambiguous weekdays/months to their futuristic relatives. Accepts a boolean,
 defaults to false.
 
 =item * C<time_zone>
@@ -517,7 +524,7 @@ recognized by L<DateTime>. Defaults to 'floating'.
 
 =item * C<daytime>
 
-A hash consisting of specific hours given for peculiar daytimes. Daytimes may be
+A hash reference consisting of customized daytime hours, which may be
 selectively changed.
 
 =back
@@ -557,16 +564,16 @@ string given.
 
 =head2 error
 
-Returns the error message if the parsing didn't succeed.
+Returns the error message if the parsing did not succeed.
 
 =head2 trace
 
-Returns a trace of methods which we're called within the Base class and
-a summary how often certain units were modified.
+Returns a trace of methods which were called within the Base class and
+a summary how often certain units have been modified.
 
 =head1 GRAMMAR
 
-The grammar handling has been rewritten to be easily extendable and hence 
+The grammar handling has been rewritten to be easily extendable and hence
 everybody is encouraged to propose sensible new additions and/or changes.
 
 See the classes C<DateTime::Format::Natural::Lang::[language_code]> if
@@ -574,7 +581,7 @@ you're intending to hack a bit on the grammar guts.
 
 =head1 EXAMPLES
 
-See the classes C<DateTime::Format::Natural::Lang::[language_code]> for a 
+See the classes C<DateTime::Format::Natural::Lang::[language_code]> for a
 overview of current valid input.
 
 =head1 CREDITS
@@ -601,6 +608,9 @@ valuable suggestions & patches:
  Jason May
  Pat Kale
  Ankur Gupta
+ Alex Bowley
+ Elliot Shank
+ Anirvan Chatterjee
 
 =head1 SEE ALSO
 
