@@ -9,7 +9,7 @@ use Date::Calc qw(Add_Delta_Days
                   Nth_Weekday_of_Month_Year
                   check_date check_time);
 
-our $VERSION = '1.17';
+our $VERSION = '1.20';
 
 use constant MORNING   => '08';
 use constant AFTERNOON => '14';
@@ -873,6 +873,20 @@ sub _day_month_year
     $self->_set_modified(3);
 }
 
+sub _count_weekday_from_now
+{
+    my $self = shift;
+    $self->_add_trace;
+    my ($count, $weekday) = @_;
+    chop $weekday if $weekday =~ /s$/;
+    my $wday = $self->{datetime}->wday;
+    my $dow  = Decode_Day_of_Week($weekday);
+    my $diff = ($wday < $dow) ? $dow - $wday : (7 - $wday) + $dow;
+    my $days = ($count - 1) * 7 + $diff;
+    $self->_add(day => $days);
+    $self->_set_modified(4);
+}
+
 sub _day_name
 {
     my $self = shift;
